@@ -1,3 +1,6 @@
+//@line 36 "/usr/obj/ports/mozilla-firefox-1.5.0.1/mozilla/browser/base/content/aboutDialog.js"
+
+var gSelectedPage = 0;
 
 function init(aEvent) 
 {
@@ -5,11 +8,13 @@ function init(aEvent)
     return;
   var userAgentField = document.getElementById("userAgent");
   userAgentField.value = navigator.userAgent;
-  
+
   var button = document.documentElement.getButton("extra2");
   button.setAttribute("label", document.documentElement.getAttribute("creditslabel"));
-  gSelectedPage = 0;
   button.addEventListener("command", switchPage, false);
+
+  document.documentElement.getButton("accept").focus();
+//@line 56 "/usr/obj/ports/mozilla-firefox-1.5.0.1/mozilla/browser/base/content/aboutDialog.js"
 }
 
 function uninit(aEvent)
@@ -28,7 +33,7 @@ function switchPage(aEvent)
 
   var iframe = document.getElementById("creditsIframe");
   if (gSelectedPage == 0) { 
-    iframe.setAttribute("src", "chrome://browser/locale/credits.html");
+    iframe.setAttribute("src", "chrome://browser/content/credits.xhtml");
     button.setAttribute("label", document.documentElement.getAttribute("aboutlabel"));
     gSelectedPage = 1;
   }
