@@ -1,3 +1,4 @@
+//@line 40 "/usr/obj/i386/mozilla-firefox-1.5.0.1/mozilla/browser/components/bookmarks/content/bookmarksManager.js"
 
 var gSearchBox;
 
@@ -8,6 +9,7 @@ function Startup()
 {
   const windowNode = document.getElementById("bookmark-window");
   const bookmarksView = document.getElementById("bookmarks-view");
+  const bookmarksFolder = document.getElementById("bookmark-folders-view");
 
   var titleString;
 
@@ -16,7 +18,9 @@ function Startup()
     var title;
     var uri = window.arguments[0];
     bookmarksView.tree.setAttribute("ref", uri);
+    bookmarksFolder.setAttribute("hidden","true")
     document.getElementById("bookmarks-search").setAttribute("hidden","true")
+    document.getElementById("bookmark-views-splitter").setAttribute("hidden","true")    
     if (uri.substring(0,5) == "find:") {
       title = BookmarksUtils.getLocaleString("search_results_title");
       // Update the windowtype so that future searches are directed 
@@ -24,20 +28,21 @@ function Startup()
       windowNode.setAttribute("windowtype", "bookmarks:searchresults");
     }
     else
-      title = BookmarksUtils.getProperty(window.arguments[0], NC_NS+"Name");
+      title = BookmarksUtils.getProperty(window.arguments[0], gNC_NS+"Name");
     
     titleString = BookmarksUtils.getLocaleString("window_title", title);
   }
   else {
     titleString = BookmarksUtils.getLocaleString("bookmarks_title", title);
     // always open the bookmark root folder
-    if (!bookmarksView.treeBoxObject.view.isContainerOpen(0))
-      bookmarksView.treeBoxObject.view.toggleOpenState(0);
+    if (!bookmarksFolder.treeBoxObject.view.isContainerOpen(0))
+      bookmarksFolder.treeBoxObject.view.toggleOpenState(0);
+    bookmarksFolder.treeBoxObject.view.selection.select(0);
   }
 
-  bookmarksView.treeBoxObject.selection.select(0);
+  bookmarksView.treeBoxObject.view.selection.select(0);
 
-  windowNode.setAttribute("title", titleString);
+  document.title = titleString;
 
   document.getElementById("CommandUpdate_Bookmarks").setAttribute("commandupdater","true");
   bookmarksView.focus();
@@ -85,7 +90,7 @@ function fillViewMenu(aEvent)
     gConstructedViewMenuSortItems = true;
   }  
 
-  const kPrefSvcContractID = "@mozilla.org/preferences;1";
+  const kPrefSvcContractID = "@mozilla.org/preferences-service;1";
   const kPrefSvcIID = Components.interfaces.nsIPrefService;
   var prefSvc = Components.classes[kPrefSvcContractID].getService(kPrefSvcIID);
   var bookmarksSortPrefs = prefSvc.getBranch("browser.bookmarks.sort.");
@@ -117,7 +122,7 @@ function onViewMenuSortItemSelected(aEvent)
 {
   var resource = aEvent.target.getAttribute("resource");
   
-  const kPrefSvcContractID = "@mozilla.org/preferences;1";
+  const kPrefSvcContractID = "@mozilla.org/preferences-service;1";
   const kPrefSvcIID = Components.interfaces.nsIPrefService;
   var prefSvc = Components.classes[kPrefSvcContractID].getService(kPrefSvcIID);
   var bookmarksSortPrefs = prefSvc.getBranch("browser.bookmarks.sort.");
@@ -186,3 +191,39 @@ function onViewMenuColumnItemSelected(aEvent)
 
   aEvent.preventBubble();
 }
+
+function onViewSelected(aEvent)
+{
+  var statusBar = document.getElementById("statusbar-text");
+  var displayValue;
+  var selection = aEvent.target.getTreeSelection();
+  var bookmarksView = document.getElementById("bookmarks-view");
+
+  if (selection.length == 0)
+      return;
+
+  if (aEvent.target.id == "bookmark-folders-view" && selection)
+    bookmarksView.tree.setAttribute("ref",selection.item[0].Value);
+      
+  if (statusBar && selection.length == 1) {
+    //protocol broken since we have unique ID...
+    //var protocol = selection.protocol[0];
+    if (selection.isContainer[0]) {// && protocol != "find" && protocol != "file") {
+      RDFC.Init(aEvent.target.db, selection.item[0]);
+      var count = 0;
+      var children = RDFC.GetElements();
+      while (children.hasMoreElements()) {
+        if (BookmarksUtils.resolveType(children.getNext()) != "BookmarkSeparator")
+          count++;
+      }
+
+      displayValue = BookmarksUtils.getLocaleString("status_foldercount", String(count));
+    }
+    else if (selection.type[0] == "Bookmark")
+      displayValue = BookmarksUtils.getProperty(selection.item[0], gNC_NS+"URL", aEvent.target.db)
+    else
+      displayValue = "";
+    statusBar.label = displayValue;
+  }
+}
+
