@@ -1,3 +1,4 @@
+//@line 38 "/usr/obj/ports/mozilla-firefox-1.5.0.1/mozilla/browser/components/history/content/history.js"
 // The history window uses JavaScript in bookmarksManager.js too.
 
 var gHistoryTree;
@@ -22,9 +23,10 @@ function HistoryCommonInit()
       document.getElementById("bydayandsite").setAttribute("checked", "true");
     else
       document.getElementById("byday").setAttribute("checked", "true");
-    gHistoryTree.focus();
-    SortInNewDirection(find_sort_direction(find_sort_column()));
-    gHistoryTree.treeBoxObject.view.selection.select(0);
+    
+    // XXXBlake we should persist the last search value
+    // If it's empty, this will do the right thing and just group by the old grouping.
+    searchHistory(gSearchBox.value);
 }
 
 function historyOnSelect()
@@ -39,8 +41,9 @@ var historyDNDObserver = {
         if (isContainer(gHistoryTree, currentIndex))
             return false;
         var builder = gHistoryTree.builder.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
-        var url = builder.getResourceAtIndex(currentIndex).Value;
-        var title = gHistoryTree.treeBoxObject.view.getCellText(currentIndex, "Name");
+        var url = builder.getResourceAtIndex(currentIndex).ValueUTF8;
+        var col = gHistoryTree.columns["Name"];
+        var title = gHistoryTree.view.getCellText(currentIndex, col);
 
         var htmlString = "<A HREF='" + url + "'>" + title + "</A>";
         aXferData.data = new TransferData();
@@ -57,39 +60,9 @@ function collapseExpand()
     gHistoryTree.treeBoxObject.view.toggleOpenState(currentIndex);
 }
 
-function onDoubleClick(event)
+function openURLIn(where)
 {
-  if (event.metaKey || event.shiftKey)
-    OpenURL(1);
-  else if (event.ctrlKey)
-    OpenURL(2, event);
-  else
-    OpenURL(0);
-}
-
-function checkURLSecurity(aURL)
-{
-  var uri = Components.classes["@mozilla.org/network/standard-url;1"].
-              createInstance(Components.interfaces.nsIURI);
-  uri.spec = aURL;
-  if (uri.schemeIs("javascript") || uri.schemeIs("data")) {
-    var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"]
-                                      .getService(Components.interfaces.nsIStringBundleService);
-    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-                                  .getService(Components.interfaces.nsIPromptService);
-    var historyBundle = strBundleService.createBundle("chrome://communicator/locale/history/history.properties");
-    var brandBundle = strBundleService.createBundle("chrome://global/locale/brand.properties");      
-    var brandStr = brandBundle.GetStringFromName("brandShortName");
-    var errorStr = historyBundle.GetStringFromName("load-js-data-url-error");
-    promptService.alert(window, brandStr, errorStr);
-    return false;
-  }
-  return true;
-}
-
-function OpenURL(aWhere, event)
-{
-  var count = gHistoryTree.treeBoxObject.view.selection.count;
+  var count = gHistoryTree.view.selection.count;
   if (count != 1)
     return;
 
@@ -98,25 +71,76 @@ function OpenURL(aWhere, event)
     return;
  
   var builder = gHistoryTree.builder.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
-  var url = builder.getResourceAtIndex(currentIndex).Value;
+  var url = builder.getResourceAtIndex(currentIndex).ValueUTF8;
   
   if (!checkURLSecurity(url)) 
     return;
 
-  if (aWhere == 0)
-    openTopWin(url);
-  else if (aWhere == 1)
-    openNewWindowWith(url, null, false);
-  else
-    openNewTabWith(url, null, event, false);     
+  openUILinkIn(url, where);
+}
+
+function openURL(aEvent)
+{
+  openURLIn(whereToOpenLink(aEvent));
+}
+
+function handleHistoryClick(aEvent)
+{
+  var tbo = gHistoryTree.treeBoxObject;
+
+  var row = { }, col = { }, obj = { };
+  tbo.getCellAt(aEvent.clientX, aEvent.clientY, row, col, obj);
+  
+  var x = { }, y = { }, w = { }, h = { };
+  tbo.getCoordsForCellItem(row.value, col.value, "image",
+                           x, y, w, h);
+  var mouseInGutter = aEvent.clientX < x.value;
+
+  if (row.value == -1 || obj.value == "twisty")
+    return;
+  var modifKey = aEvent.shiftKey || aEvent.ctrlKey || aEvent.altKey || 
+                 aEvent.metaKey  || aEvent.button == 1;
+  if (!modifKey && tbo.view.isContainer(row.value)) {
+    tbo.view.toggleOpenState(row.value);
+    return;
+  }
+  if (!mouseInGutter && 
+      aEvent.originalTarget.localName == "treechildren" && 
+      (aEvent.button == 0 || aEvent.button == 1)) {
+    // Clear all other selection since we're loading a link now. We must
+    // do this *before* attempting to load the link since openURL uses
+    // selection as an indication of which link to load. 
+    tbo.view.selection.select(row.value);
+
+    openURL(aEvent);
+  }
+}
+
+function checkURLSecurity(aURL)
+{
+  var uri = Components.classes["@mozilla.org/network/io-service;1"]
+                      .getService(Components.interfaces.nsIIOService)
+                      .newURI(aURL, null, null);
+
+  if (uri.schemeIs("javascript") || uri.schemeIs("data")) {
+    var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"]
+                                      .getService(Components.interfaces.nsIStringBundleService);
+    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
+                                  .getService(Components.interfaces.nsIPromptService);
+    var historyBundle = strBundleService.createBundle("chrome://global/locale/history/history.properties");
+    var brandBundle = strBundleService.createBundle("chrome://branding/locale/brand.properties");      
+    var brandStr = brandBundle.GetStringFromName("brandShortName");
+    var errorStr = historyBundle.GetStringFromName("load-js-data-url-error");
+    promptService.alert(window, brandStr, errorStr);
+    return false;
+  }
+  return true;
 }
 
 function SortBy(sortKey)
 {
-  // XXXBlake Welcome to the end of the world of Lame.
-  // You can go no further. You are standing on the edge -- teetering, even.
-  // Look on the bright side: you can rest assured that no code you see in the future
-  // will even come close to the lameness of the code below.
+  // We set the sortDirection to the one before we actually want it to be in the
+  // cycle list, since cycleHeader cycles it forward before doing the sort.
 
   var sortDirection;
   switch(sortKey) {
@@ -125,82 +149,108 @@ function SortBy(sortKey)
       sortDirection = "ascending";
       break;
     case "name":
-      sortKey = "rdf:http://home.netscape.com/NC-rdf#Name";
+      sortKey = "rdf:http://home.netscape.com/NC-rdf#Name?sort=true";
       sortDirection = "natural";
       break;
     case "lastvisited":
       sortKey = "rdf:http://home.netscape.com/NC-rdf#Date";
       sortDirection = "ascending";
       break;
-    case "day":
-      sortKey = "rdf:http://home.netscape.com/NC-rdf#DayFolderIndex";
-      sortDirection = "natural";
-      break;
     default:
       return;    
   }
   var col = document.getElementById("Name");
   col.setAttribute("sort", sortKey);
   col.setAttribute("sortDirection", sortDirection);
-  gHistoryTree.treeBoxObject.view.cycleHeader(sortKey, col);
+  var column = gHistoryTree.columns.getColumnFor(col);
+  gHistoryTree.view.cycleHeader(column);
 }
 
+function IsFindResource(uri)
+{
+  return (uri.substr(0, 5) == "find:");
+}
+    
 function GroupBy(groupingType)
 {
+  var isFind = IsFindResource(groupingType);
+  if (!isFind) {
     gHistoryGrouping = groupingType;
-    switch(groupingType) {
-    case "none":
-        gHistoryTree.setAttribute("ref", "NC:HistoryRoot");
-        break;
-    case "site":
-        // xxx for now
-        gHistoryTree.setAttribute("ref", "NC:HistoryRoot");
-        SortBy("name");
-        break;
-    case "dayandsite":
-        gHistoryTree.setAttribute("ref", "NC:HistoryByDateAndSite");
-        SortBy("dayandsite");
-        break;
-    case "visited":
-        gHistoryTree.setAttribute("ref", "NC:HistoryRoot");
-        SortBy("visited");
-        break;
-    case "lastvisited":
-        gHistoryTree.setAttribute("ref", "NC:HistoryRoot");
-        SortBy("lastvisited");
-        break;
-    case "day":
-    default:
-        gHistoryTree.setAttribute("ref", "NC:HistoryByDate");
-        SortBy("day");
-        break;
-    }
     gSearchBox.value = "";
+  }
+  switch(groupingType) {
+    case "site":
+      gHistoryTree.setAttribute("ref", "NC:HistoryRoot");
+      break;
+    case "dayandsite":
+      gHistoryTree.setAttribute("ref", "NC:HistoryByDateAndSite");
+      break;
+    case "visited":
+      gHistoryTree.setAttribute("ref", "NC:HistoryRoot");
+      break;
+    case "lastvisited":
+      gHistoryTree.setAttribute("ref", "NC:HistoryRoot");
+      break;
+    case "day":
+      gHistoryTree.setAttribute("ref", "NC:HistoryByDate");
+      break;
+    default:
+      gHistoryTree.setAttribute("ref", groupingType);
+  }
+  Sort(isFind? gHistoryGrouping : groupingType);
+}
+
+function Sort(groupingType)
+{
+  switch(groupingType) {
+    case "site":
+    case "dayandsite":
+    case "day":
+      SortBy("name");
+      break;
+    case "lastvisited":
+      SortBy("lastvisited");
+      break;
+    case "visited":
+      SortBy("visited");
+      break;
+  }
 }
 
 function historyAddBookmarks()
 {
-  var count = gHistoryTree.treeBoxObject.view.selection.count;
+  var count = gHistoryTree.view.selection.count;
   if (count != 1)
     return;
   
   var currentIndex = gHistoryTree.currentIndex;
   var builder = gHistoryTree.builder.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
-  var url = builder.getResourceAtIndex(currentIndex).Value;
+  var url = builder.getResourceAtIndex(currentIndex).ValueUTF8;
   
   //XXXBlake don't use getCellText
-  var title = gHistoryTree.treeBoxObject.view.getCellText(currentIndex, "Name");
+  var col = gHistoryTree.columns["Name"];
+  var title = gHistoryTree.view.getCellText(currentIndex, col);
   BookmarksUtils.addBookmark(url, title, undefined);
 }
 
 function historyCopyLink()
 {
-  dump("Not yet implemented!");
+  var builder = gHistoryTree.builder.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
+  var url = builder.getResourceAtIndex(gHistoryTree.currentIndex).ValueUTF8;
+  var clipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
+                            .getService(Components.interfaces.nsIClipboardHelper );
+  clipboard.copyString(url);
 }
 
-function buildContextMenu()
+function buildContextMenu(aEvent)
 {
-  var count = gHistoryTree.treeBoxObject.view.selection.count;
+  // if nothing is selected, bail and don't show a context menu
+  var count = gHistoryTree.view.selection.count;
+  if (count != 1) {
+    aEvent.preventDefault();
+    return;
+  }
+
   var openItem = document.getElementById("miOpen");
   var openItemInNewWindow = document.getElementById("miOpenInNewWindow");
   var openItemInNewTab = document.getElementById("miOpenInNewTab");
@@ -210,60 +260,45 @@ function buildContextMenu()
   var sep2 = document.getElementById("post-bookmarks-separator");
   var expandItem = document.getElementById("miExpand");
   var collapseItem = document.getElementById("miCollapse");
-  if (count > 1) {
+
+  var currentIndex = gHistoryTree.currentIndex;
+  if ((gHistoryGrouping == "day" || gHistoryGrouping == "dayandsite")
+      && isContainer(gHistoryTree, currentIndex)) {
     openItem.hidden = true;
     openItemInNewWindow.hidden = true;
     openItemInNewTab.hidden = true;
     bookmarkItem.hidden = true;
     copyLocationItem.hidden = true;
     sep1.hidden = true;
-    sep2.hidden = true;
-    expandItem.hidden = true;    
-    collapseItem.hidden = true;    
-  }
-  else {
-    var currentIndex = gHistoryTree.currentIndex;
-    if ((gHistoryGrouping == "day" || gHistoryGrouping == "dayandsite")
-        && isContainer(gHistoryTree, currentIndex)) {
-      openItem.hidden = true;
-      openItemInNewWindow.hidden = true;
-      openItemInNewTab.hidden = true;
-      bookmarkItem.hidden = true;
-      copyLocationItem.hidden = true;
-      sep1.hidden = true;
-      sep2.hidden = false;
-      if (isContainerOpen(gHistoryTree, currentIndex)) {
-        expandItem.hidden = true;
-        collapseItem.hidden = false;
-      } else {
-        expandItem.hidden = false;
-        collapseItem.hidden = true;
-      }
-    }
-    else {
-      openItem.hidden = false;
-      openItemInNewWindow.hidden = false;
-      openItemInNewTab.hidden = false;
-      bookmarkItem.hidden = false;
-      copyLocationItem.hidden = false;
-      sep1.hidden = false;
-      sep2.hidden = false;
+    sep2.hidden = false;
+    if (isContainerOpen(gHistoryTree, currentIndex)) {
       expandItem.hidden = true;
+      collapseItem.hidden = false;
+    } else {
+      expandItem.hidden = false;
       collapseItem.hidden = true;
     }
   }
+  else {
+    openItem.hidden = false;
+    openItemInNewWindow.hidden = false;
+    openItemInNewTab.hidden = false;
+    bookmarkItem.hidden = false;
+    copyLocationItem.hidden = false;
+    sep1.hidden = false;
+    sep2.hidden = false;
+    expandItem.hidden = true;
+    collapseItem.hidden = true;
+  }
 }
 
 function searchHistory(aInput)
 {
-   if (!aInput) 
+   if (aInput == "") {
      GroupBy(gHistoryGrouping);
-   else
-     gHistoryTree.setAttribute("ref",
-                               "find:datasource=history&match=Name&method=contains&text=" + encodeURIComponent(aInput));
+     return;
+   }
+   
+   GroupBy("find:datasource=history&match=Name&method=contains&text=" + encodeURIComponent(aInput));     
  }
 
-function onUnload()
-{
-  return;
-}
