@@ -1,3 +1,5 @@
+//@line 39 "/usr/obj/ports/mozilla-firefox-1.5.0.1/mozilla/toolkit/mozapps/downloads/content/DownloadProgressListener.js"
+
 var gStrings = [];
 const interval = 500; // Update every 500 milliseconds.
 
@@ -9,6 +11,10 @@ function DownloadProgressListener (aDocument, aStringBundle)
   this._statusFormatKBMB = aStringBundle.getString("statusFormatKBMB");
   this._statusFormatKBKB = aStringBundle.getString("statusFormatKBKB");
   this._statusFormatMBMB = aStringBundle.getString("statusFormatMBMB");
+  this._statusFormatUnknownMB = aStringBundle.getString("statusFormatUnknownMB");
+  this._statusFormatUnknownKB = aStringBundle.getString("statusFormatUnknownKB");
+  this._remain = aStringBundle.getString("remain");
+  this._unknownFilesize = aStringBundle.getString("unknownFilesize");
   this._longTimeFormat = aStringBundle.getString("longTimeFormat");
   this._shortTimeFormat = aStringBundle.getString("shortTimeFormat");
   
@@ -25,7 +31,7 @@ DownloadProgressListener.prototype =
   onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus, aDownload)
   {
     if (aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_STOP) {
-      var aDownloadID = aDownload.target.path;
+      var aDownloadID = aDownload.targetFile.path;
       var download = this.doc.getElementById(aDownloadID);
       if (download)
         download.setAttribute("status", "");
@@ -52,11 +58,11 @@ DownloadProgressListener.prototype =
     this.elapsed = now - (aDownload.startTime / 1000);
     var rate; // aCurTotalProgress/sec
     if (this.elapsed)
-      rate = (aCurTotalProgress * 1000) / this.elapsed;
+      rate = (aCurTotalProgress * 1024) / this.elapsed;
     else
       rate = 0;
 
-    var aDownloadID = aDownload.target.path;
+    var aDownloadID = aDownload.targetFile.path;
     var download = this.doc.getElementById(aDownloadID);
 
     // Calculate percentage.
@@ -130,10 +136,10 @@ DownloadProgressListener.prototype =
       var rem = (aMaxTotalProgress - aCurTotalProgress) / rate;
       rem = parseInt(rem + .5);
       
-      status = this._replaceInsert(status, 3, this._formatSeconds(rem, this.doc));
+      status = this._replaceInsert(status, 3, this._formatSeconds(rem, this.doc) + " " + this._remain);
     }
     else
-      status = this._replaceInsert(status, 3, "???");
+      status = this._replaceInsert(status, 3, this._unknownFilesize);
     
     if (download)
       download.setAttribute("status", status);
@@ -170,24 +176,33 @@ DownloadProgressListener.prototype =
   // x, >= 1MB  y >= 1MB        x of y MB
   _formatKBytes: function (aKBytes, aTotalKBytes)
   {
-    var progressHasMB = parseInt(aKBytes/1000) > 0;
-    var totalHasMB = parseInt(aTotalKBytes/1000) > 0;
+    var progressHasMB = parseInt(aKBytes/1024) > 0;
+    var totalHasMB = parseInt(aTotalKBytes/1024) > 0;
     
     var format = "";
     if (!progressHasMB && !totalHasMB) {
-      format = this._statusFormatKBKB;
-      format = this._replaceInsert(format, 1, aKBytes);
-      format = this._replaceInsert(format, 2, aTotalKBytes);
+      if (!aTotalKBytes) {
+      	 format = this._statusFormatUnknownKB;
+        format = this._replaceInsert(format, 1, aKBytes);
+      } else {
+        format = this._statusFormatKBKB;
+        format = this._replaceInsert(format, 1, aKBytes);
+        format = this._replaceInsert(format, 2, aTotalKBytes);
+      }
     }
     else if (progressHasMB && totalHasMB) {
       format = this._statusFormatMBMB;
-      format = this._replaceInsert(format, 1, (aKBytes / 1000).toFixed(1));
-      format = this._replaceInsert(format, 2, (aTotalKBytes / 1000).toFixed(1));
+      format = this._replaceInsert(format, 1, (aKBytes / 1024).toFixed(1));
+      format = this._replaceInsert(format, 2, (aTotalKBytes / 1024).toFixed(1));
     }
     else if (totalHasMB && !progressHasMB) {
       format = this._statusFormatKBMB;
       format = this._replaceInsert(format, 1, aKBytes);
-      format = this._replaceInsert(format, 2, (aTotalKBytes / 1000).toFixed(1));
+      format = this._replaceInsert(format, 2, (aTotalKBytes / 1024).toFixed(1));
+    }
+    else if (progressHasMB && !totalHasMB) {
+      format = this._statusFormatUnknownMB;
+      format = this._replaceInsert(format, 1, (aKBytes / 1024).toFixed(1));
     }
     else {
       // This is an undefined state!
@@ -221,9 +236,6 @@ DownloadProgressListener.prototype =
     result = this._replaceInsert(result, 3, secs);
 
     return result;
-  },
+  }
   
-};
-
-
- 
+}; 
