@@ -1,5 +1,5 @@
+//@line 41 "/usr/obj/ports/mozilla-firefox-1.5.0.1/mozilla/toolkit/mozapps/downloads/content/downloads.js"
 
- 
 ///////////////////////////////////////////////////////////////////////////////
 // Globals
 
@@ -89,7 +89,23 @@ function downloadCompleted(aDownload)
     
     rdfc.Init(db, rdf.GetResource("NC:DownloadsRoot"));
 
-    var id = aDownload.target.path;
+    var id = aDownload.targetFile.path;
+    
+    // getTypeFromFile fails if it can't find a type for this file. Handle this gracefully.
+    try {
+      // Refresh the icon, so that executable icons are shown.
+      var mimeService = Components.classes["@mozilla.org/uriloader/external-helper-app-service;1"].getService(Components.interfaces.nsIMIMEService);
+      var contentType = mimeService.getTypeFromFile(aDownload.targetFile);
+
+      var listItem = document.getElementById(id);
+      var oldImage = listItem.getAttribute("image");
+      // I tack on the content-type here as a hack to bypass the cache which seems
+      // to be interfering despite the fact the image has 'validate="always"' set
+      // on it. 
+      listItem.setAttribute("image", oldImage + "&contentType=" + contentType);
+    } catch (e) {
+    }
+    
     var dlRes = rdf.GetUnicodeResource(id);
   
     var insertIndex = gDownloadManager.activeDownloadCount + 1;
@@ -111,8 +127,11 @@ function downloadCompleted(aDownload)
         break;
       }
     }
+
+    gDownloadViewController.onCommandUpdate();
+
     if (gActiveDownloads.length == 0)
-      window.title = document.documentElement.getAttribute("statictitle");    
+      document.title = document.documentElement.getAttribute("statictitle");
   }
   catch (e) {
   }
@@ -159,9 +178,14 @@ var gDownloadObserver = {
       downloadCompleted(dl);
       break;
     case "dl-start":
+      // whenever a new download starts, it is added to the top, so switch the
+      // view to it
+      gDownloadsView.selectedIndex = 0;
+
       // Add this download to the percentage average tally
       var dl = aSubject.QueryInterface(Components.interfaces.nsIDownload);
       gActiveDownloads.push(dl);
+
       break;
     case "xpinstall-download-started":
       var windowArgs = aSubject.QueryInterface(Components.interfaces.nsISupportsArray);
@@ -187,6 +211,8 @@ var gDownloadObserver = {
 
 function onDownloadCancel(aEvent)
 {
+  var selectedIndex = gDownloadsView.selectedIndex;
+
   gDownloadManager.cancelDownload(aEvent.target.id);
 
   setRDFProperty(aEvent.target.id, "DownloadAnimated", "false");
@@ -197,49 +223,79 @@ function onDownloadCancel(aEvent)
   // employed by the helper app service isn't fully accessible yet... should be fixed...
   // talk to bz...)
   // the upshot is we have to delete the file if it exists. 
-  var f = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
-  f.initWithPath(aEvent.target.id);
+  var f = getLocalFileFromNativePathOrUrl(aEvent.target.id);
+
   if (f.exists()) 
-    f.remove(true);
+    f.remove(false);
 
   gDownloadViewController.onCommandUpdate();
+
+  // now reset the richlistbox
+  gDownloadsView.clearSelection();
+  var rowCount = gDownloadsView.getRowCount();
+  if (selectedIndex >= rowCount)
+    gDownloadsView.selectedIndex = rowCount - 1;
+  else
+    gDownloadsView.selectedIndex = selectedIndex;
 }
 
 function onDownloadPause(aEvent)
 {
+  var selectedIndex = gDownloadsView.selectedIndex;
+
   var uri = aEvent.target.id;
   gDownloadManager.pauseDownload(uri);
   setRDFProperty(uri, "DownloadStatus", aEvent.target.getAttribute("status-internal"));
   setRDFProperty(uri, "ProgressPercent", aEvent.target.getAttribute("progress"));
+
+  // now reset the richlistbox
+  gDownloadsView.clearSelection();
+  var rowCount = gDownloadsView.getRowCount();
+  if (selectedIndex >= rowCount)
+    gDownloadsView.selectedIndex = rowCount - 1;
+  else
+    gDownloadsView.selectedIndex = selectedIndex;
 }
 
 function onDownloadResume(aEvent)
 {
+  var selectedIndex = gDownloadsView.selectedIndex;
+
   gDownloadManager.resumeDownload(aEvent.target.id);
+
+  // now reset the richlistbox
+  gDownloadsView.clearSelection();
+  var rowCount = gDownloadsView.getRowCount();
+  if (selectedIndex >= rowCount)
+    gDownloadsView.selectedIndex = rowCount - 1;
+  else
+    gDownloadsView.selectedIndex = selectedIndex;
 }
 
 function onDownloadRemove(aEvent)
 {
   if (aEvent.target.removable) {
+    var selectedIndex = gDownloadsView.selectedIndex;
     gDownloadManager.removeDownload(aEvent.target.id);
-    
     gDownloadViewController.onCommandUpdate();
   }
 }
 
 function onDownloadShow(aEvent)
 {
-  var f = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
-  f.initWithPath(aEvent.target.id);
+  var f = getLocalFileFromNativePathOrUrl(aEvent.target.id);
 
   if (f.exists()) {
-    // on unix, open a browser window rooted at the parent
-    var parent = f.parent;
-    if (parent) {
-      var pref = Components.classes["@mozilla.org/preferences-service;1"]
-                          .getService(Components.interfaces.nsIPrefBranch);
-      var browserURL = pref.getCharPref("browser.chromeURL");                          
-      window.openDialog(browserURL, "_blank", "chrome,all,dialog=no", parent.path);
+    try {
+      f.reveal();
+    } catch (ex) {
+      // if reveal failed for some reason (eg on unix it's not currently
+      // implemented), send the file: URL window rooted at the parent to 
+      // the OS handler for that protocol
+      var parent = f.parent;
+      if (parent) {
+        openExternal(parent);
+      }
     }
   }
   else {
@@ -248,7 +304,7 @@ function onDownloadShow(aEvent)
   
     var strings = document.getElementById("downloadStrings");
     var name = aEvent.target.getAttribute("target");
-    var message = strings.getFormattedString("fileDoesNotExistShowErrorUnix", [name, appName]);
+    var message = strings.getFormattedString("fileDoesNotExistError", [name, appName]);
     var title = strings.getFormattedString("fileDoesNotExistShowTitle", [name]);
 
     var promptSvc = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
@@ -258,14 +314,13 @@ function onDownloadShow(aEvent)
 
 function onDownloadOpen(aEvent)
 {
+  if (aEvent.type == "dblclick" && aEvent.button != 0)
+    return;
   var download = aEvent.target;
   if (download.localName == "download") {
     if (download.openable) {
-      var f = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
-      f.initWithPath(aEvent.target.id);
-
+      var f = getLocalFileFromNativePathOrUrl(aEvent.target.id);
       if (f.exists()) {
-        // XXXben security check!  
         if (f.isExecutable()) {
           var dontAsk = false;
           var pref = Components.classes["@mozilla.org/preferences-service;1"]
@@ -293,7 +348,13 @@ function onDownloadOpen(aEvent)
               pref.setBoolPref(PREF_BDM_ALERTONEXEOPEN, !checkbox.value);              
           }        
         }
-        f.launch();
+        try {
+          f.launch();
+        } catch (ex) {
+          // if launch fails, try sending it through the system's external
+          // file: URL handler
+          openExternal(f);
+        }
       }
       else {
         var brandStrings = document.getElementById("brandStrings");
@@ -301,7 +362,7 @@ function onDownloadOpen(aEvent)
       
         var strings = document.getElementById("downloadStrings");
         var name = aEvent.target.getAttribute("target");
-        var message = strings.getFormattedString("fileDoesNotExistOpenError", [name, appName]);
+        var message = strings.getFormattedString("fileDoesNotExistError", [name, appName]);
 
         var title = strings.getFormattedString("fileDoesNotExistOpenTitle", [name]);
 
@@ -338,46 +399,57 @@ function onDownloadRetry(aEvent)
   var download = aEvent.target;
   if (download.localName == "download") {
     var src = getRDFProperty(download.id, "URL");
-    var f = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
-    f.initWithPath(aEvent.target.id);
+    var f = getLocalFileFromNativePathOrUrl(aEvent.target.id);
     saveURL(src, f, null, true, true);
   }
-  
+
   gDownloadViewController.onCommandUpdate();
+
+  // retry always places the item in the first spot
+  gDownloadsView.selectedIndex = 0;
 }
 
 // This is called by the progress listener. We don't actually use the event
 // system here to minimize time wastage. 
-var gLastComputedMean = 0;
+var gLastComputedMean = -1;
+var gLastActiveDownloads = 0;
 function onUpdateProgress()
 {
   var numActiveDownloads = gActiveDownloads.length;
   if (numActiveDownloads == 0) {
-    window.title = document.documentElement.getAttribute("statictitle");
-    gLastComputedMean = 0;
+    document.title = document.documentElement.getAttribute("statictitle");
+    gLastComputedMean = -1;
     return;
   }
     
   var mean = 0;
+  var base = 0;
+  var dl = null;
   for (var i = 0; i < numActiveDownloads; ++i) {
-    var dl = gActiveDownloads[i];
-    var progress = dl.percentComplete;
-    if (progress < 100)
-      mean += progress;
+    dl = gActiveDownloads[i];
+
+    // gActiveDownloads is screwed so it's possible 
+    // to have more files than we're really downloading.
+    // The good news is that those files have size==0.
+    // Same with files with unknown size. Their size==0.
+    if (dl.percentComplete < 100 && dl.size > 0) {
+      mean += dl.amountTransferred;
+      base += dl.size;
+    }
   }
 
-  mean = Math.round(mean / numActiveDownloads);
-  
-  // At the end of a download, progress is set from 100% to 0% for 
-  // some reason. We can identify this case because at this point the
-  // mean progress will be zero but the last computed mean will be
-  // greater than zero. 
-  if (mean == 0 && gLastComputedMean > 0) {
-    window.title = document.documentElement.getAttribute("statictitle");
-    return;
+  // we're not downloading anything at the moment,
+  // but we already downloaded something.
+  if (base == 0) {
+    mean = 100;
+  } else {
+    mean = Math.floor((mean / base) * 100);
   }
-  if (mean != gLastComputedMean) {
+
+  if (mean != gLastComputedMean || gLastActiveDownloads != numActiveDownloads) {
     gLastComputedMean = mean;
+    gLastActiveDownloads = numActiveDownloads;
+    
     var strings = document.getElementById("downloadStrings");
     
     var title;
@@ -386,7 +458,7 @@ function onUpdateProgress()
     else
       title = strings.getFormattedString("downloadsTitle", [mean]);
 
-    window.title = title;
+    document.title = title;
   }
 }
 
@@ -425,6 +497,12 @@ function Startup()
   gDownloadsView.addEventListener("download-properties",  onDownloadProperties, false);
   gDownloadsView.addEventListener("dblclick",             onDownloadOpen,       false);
   
+  // Set up AutoDownload display area
+  initAutoDownloadDisplay();
+  var pbi = Components.classes["@mozilla.org/preferences-service;1"]
+                      .getService(Components.interfaces.nsIPrefBranch2);
+  pbi.addObserver("browser.download.", gDownloadPrefObserver, false);
+  
   // Handlers for events generated by the Download Manager (download events)
   var observerService = Components.classes[kObserverServiceProgID]
                                   .getService(Components.interfaces.nsIObserverService);
@@ -454,8 +532,9 @@ function Startup()
   // Finally, update the UI. 
   gDownloadsView.database.AddDataSource(gDownloadManager.datasource);
   gDownloadsView.builder.rebuild();
-  
-  gDownloadsView.focus();
+
+  // downloads can finish before Startup() does, so check if the window should close
+  autoClose();
 }
 
 function Shutdown() 
@@ -465,6 +544,10 @@ function Shutdown()
   // Assert the current progress for all the downloads in case the window is reopened
   gDownloadManager.saveState();
 
+  var pbi = Components.classes["@mozilla.org/preferences-service;1"]
+                      .getService(Components.interfaces.nsIPrefBranch2);
+  pbi.removeObserver("browser.download.", gDownloadPrefObserver);
+
   var observerService = Components.classes[kObserverServiceProgID]
                                   .getService(Components.interfaces.nsIObserverService);
   observerService.removeObserver(gDownloadObserver, "dl-done");
@@ -472,6 +555,7 @@ function Shutdown()
   observerService.removeObserver(gDownloadObserver, "dl-failed");  
   observerService.removeObserver(gDownloadObserver, "dl-start");  
   observerService.removeObserver(gDownloadObserver, "xpinstall-download-started");  
+  observerService.removeObserver(gDownloadObserver, "xpinstall-dialog-close");
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -486,18 +570,21 @@ var XPInstallDownloadManager = {
     var tempDir = fileLocator.get("TmpD", Components.interfaces.nsIFile);
 
     var mimeService = Components.classes["@mozilla.org/uriloader/external-helper-app-service;1"].getService(Components.interfaces.nsIMIMEService);
-    
+
+    var IOService = Components.classes["@mozilla.org/network/io-service;1"]
+                              .getService(Components.interfaces.nsIIOService);
+
     var xpinstallManager = gDownloadManager.QueryInterface(Components.interfaces.nsIXPInstallManagerUI);
 
     var xpiString = "";
+
     for (var i = 0; i < numXPInstallItems;) {
       // Pretty Name
       var displayName = aParams.GetString(i++);
       
       // URI
-      var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);
-      uri.spec = aParams.GetString(i++);
-      
+      var uri = IOService.newURI(aParams.GetString(i++), null, null);
+
       var iconURL = aParams.GetString(i++);
       
       // Local File Target
@@ -517,8 +604,9 @@ var XPInstallDownloadManager = {
       if (!iconURL) 
         iconURL = "chrome://mozapps/skin/xpinstall/xpinstallItemGeneric.png";
       
+      var targetUrl = makeFileURI(localTarget);
       var download = gDownloadManager.addDownload(Components.interfaces.nsIXPInstallManagerUI.DOWNLOAD_TYPE_INSTALL, 
-                                                  uri, localTarget, displayName, iconURL, mimeInfo, 0, null);
+                                                  uri, targetUrl, displayName, iconURL, mimeInfo, 0, null);
       
       // Advance the enumerator
       var certName = aParams.GetString(i++);
@@ -543,14 +631,14 @@ var gContextMenus = [
 function buildContextMenu(aEvent)
 {
   if (aEvent.target.id != "downloadContextMenu")
-    return;
+    return false;
     
   var popup = document.getElementById("downloadContextMenu");
   while (popup.hasChildNodes())
     popup.removeChild(popup.firstChild);
   
-  if (gDownloadsView.selected) {
-    var idx = parseInt(gDownloadsView.selected.getAttribute("state"));
+  if (gDownloadsView.selectedItem) {
+    var idx = parseInt(gDownloadsView.selectedItem.getAttribute("state"));
     if (idx < 0)
       idx = 0;
     
@@ -643,7 +731,149 @@ function onDownloadShowOptions()
 
 function onDownloadShowInfo()
 {
-  if (gDownloadsView.selected)
-    fireEventForElement(gDownloadsView.selected, "properties");
+  if (gDownloadsView.selectedItem)
+    fireEventForElement(gDownloadsView.selectedItem, "properties");
 }
 
+function initAutoDownloadDisplay()
+{
+  var pref = Components.classes["@mozilla.org/preferences-service;1"]
+                       .getService(Components.interfaces.nsIPrefBranch);
+
+  var autodownload = pref.getBoolPref("browser.download.useDownloadDir");  
+  if (autodownload) {
+    var autodownloadInfo = document.getElementById("autodownloadInfo");
+    autodownloadInfo.hidden = false;
+    var autodownloadSpring = document.getElementById("autodownloadSpring");
+    autodownloadSpring.hidden = true; 
+
+    function getSpecialFolderKey(aFolderType) 
+    {
+//@line 803 "/usr/obj/ports/mozilla-firefox-1.5.0.1/mozilla/toolkit/mozapps/downloads/content/downloads.js"
+      return "Home";
+    }
+    
+    function getDownloadsFolder(aFolder)
+    {
+      var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"]
+                                  .getService(Components.interfaces.nsIProperties);
+      var dir = fileLocator.get(getSpecialFolderKey(aFolder), Components.interfaces.nsILocalFile);
+      
+      var bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
+                              .getService(Components.interfaces.nsIStringBundleService);
+      bundle = bundle.createBundle("chrome://mozapps/locale/downloads/unknownContentType.properties");
+
+      var description = bundle.GetStringFromName("myDownloads");
+      if (aFolder != "Desktop")
+        dir.append(description);
+        
+      return dir;
+    }
+
+    var displayName = null;
+    var folder;
+    switch (pref.getIntPref("browser.download.folderList")) {
+    case 0:
+      folder = getDownloadsFolder("Desktop");
+      displayName = document.getElementById("downloadStrings").getString("displayNameDesktop");
+      break;
+    case 1:
+      folder = getDownloadsFolder("Downloads");
+      break;
+    case 2: 
+      folder = pref.getComplexValue("browser.download.dir", Components.interfaces.nsILocalFile);
+      break;
+    }
+
+    if (folder) {    
+      var ioServ = Components.classes["@mozilla.org/network/io-service;1"]
+                              .getService(Components.interfaces.nsIIOService);
+      var fph = ioServ.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
+      var mozIconURI = "moz-icon://" +  fph.getURLSpecFromFile(folder) + "?size=16";
+      var folderIcon = document.getElementById("saveToFolder")
+      folderIcon.image = mozIconURI;
+      
+      var folderName = document.getElementById("saveToFolder");
+      folderName.label = displayName || folder.leafName;
+      folderName.setAttribute("path", folder.path);
+    }
+  }
+  else {
+    var autodownloadInfo = document.getElementById("autodownloadInfo");
+    autodownloadInfo.hidden = true;
+    var autodownloadSpring = document.getElementById("autodownloadSpring");
+    autodownloadSpring.hidden = false; 
+  }
+}
+
+var gDownloadPrefObserver = {
+  observe: function (aSubject, aTopic, aData)
+  {
+    if (aTopic == "nsPref:changed") {
+      switch(aData) {
+      case "browser.download.folderList":
+      case "browser.download.useDownloadDir":
+      case "browser.download.dir":
+        initAutoDownloadDisplay();
+      }
+    }
+  }
+};
+
+function onDownloadShowFolder()
+{
+  var folderName = document.getElementById("saveToFolder");
+  var dir = getLocalFileFromNativePathOrUrl(folderName.getAttribute("path"));
+  if (!dir.exists())
+   dir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0755);
+
+  try {
+    dir.reveal();
+  } catch (ex) {
+    // if nsILocalFile::Reveal failed (eg it currently just returns an
+    // error on unix), just open the folder in a browser window
+    openExternal(dir);
+  }
+}
+
+function openExternal(aFile)
+{
+  var uri = Components.classes["@mozilla.org/network/io-service;1"]
+                      .getService(Components.interfaces.nsIIOService)
+                      .newFileURI(aFile);
+
+  var protocolSvc = 
+      Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
+                .getService(Components.interfaces.nsIExternalProtocolService);
+
+  protocolSvc.loadUrl(uri);
+
+  return;
+}
+
+// we should be using real URLs all the time, but until 
+// bug 239948 is fully fixed, this will do...
+function getLocalFileFromNativePathOrUrl(aPathOrUrl)
+{
+  if (aPathOrUrl.substring(0,7) == "file://") {
+
+    // if this is a URL, get the file from that
+    ioSvc = Components.classes["@mozilla.org/network/io-service;1"]
+      .getService(Components.interfaces.nsIIOService);
+
+    // XXX it's possible that using a null char-set here is bad
+    const fileUrl = ioSvc.newURI(aPathOrUrl, null, null).
+      QueryInterface(Components.interfaces.nsIFileURL);
+    return fileUrl.file.clone().
+      QueryInterface(Components.interfaces.nsILocalFile);
+
+  } else {
+
+    // if it's a pathname, create the nsILocalFile directly
+    var f = Components.classes["@mozilla.org/file/local;1"].
+      createInstance(Components.interfaces.nsILocalFile);
+    f.initWithPath(aPathOrUrl);
+
+    return f;
+  }
+}
