@@ -1,3 +1,4 @@
+//@line 38 "/usr/obj/i386/mozilla-firefox-1.5.0.1/mozilla/toolkit/mozapps/downloads/content/helperApps.js"
 
 var gRDF;    
 
@@ -105,7 +106,7 @@ HelperApps.prototype = {
       while (extEnumerator.hasMore()) {
           entry.addExtension(extEnumerator.getNext());
       }
-      entry.description = aMIMEInfo.Description;
+      entry.description = aMIMEInfo.description;
       entry.appDisplayName = "";
     }
     
@@ -197,7 +198,7 @@ HelperApps.prototype = {
           }
         }
         else if (aProperty.EqualsNode(this._fileTypeArc)) {
-          if (typeInfo.Description == "") {
+          if (typeInfo.description == "") {
             try {
               var literal = bundle.getFormattedString("fileEnding", [typeInfo.primaryExtension.toUpperCase()]);
               return gRDF.GetLiteral(literal);
@@ -208,7 +209,7 @@ HelperApps.prototype = {
               return gRDF.GetLiteral(typeInfo.MIMEType);
             }
           }
-          return gRDF.GetLiteral(typeInfo.Description);
+          return gRDF.GetLiteral(typeInfo.description);
         }
         else if (aProperty.EqualsNode(this._fileHandlerArc)) {
           var handler = this.GetTarget(aSource, this._handlerPropArc, true);
